import { Graphic } from "./../Graphic";
import { Color, Enum } from "./../Tool";
import { IButtonProps, ITextProps, IShapeProps } from "./../Interface";


/**
 * This Graphic helpers let you create a button easily
 */
export class Button extends Graphic {

    /* ATTRIBUTES */

    /**
     * Properties of a button
     */
    props: IButtonProps;


    /* LIFECYCLE */

    /**
     * @constructor
     */
    constructor () {
        super();

        this.setProps({
            label: "",
            labelCentered: false,
            paddingLeft: 0,
            paddingRight: 0,
            paddingTop: 0,
            paddingBottom: 0
        });
    }

    /**
     * @initialize
     * @override
     */
    initialize (props) {
        super.initialize(props);

        this.shape("shape", {
            stroke: this.context.theme.primaryColor,
            strokeThickness: 2,
            fill: Color.black,
            fillAlpha: 0.5,
            radius: 5

        }, {
            hover: {
                fill: Color.cyan900,
                fillAlpha: 1
            },

            disabled: {
                fill: Color.grey900,
                fillAlpha: 0.7,
                stroke: Color.grey400
            }

        }).text("label", {
            text: this.props.label,
            fill: Color.white,
            dropShadow: true,
            dropShadowColor: Color.black,
            dropShadowAngle: 80,
            dropShadowBlur: 30,
            dropShadowDistance: 1

        }, {
            disabled: {
                fill: Color.grey400
            }
        });

        this.connect(this.signals.propChange, ["label", "labelCentered"], this.onSizeChange.bind(this))
            .connect(this.getGraphicItem("label").signals.propChange, ["width", "height"], this.onSizeChange.bind(this));
    }


    /* EVENTS */

    /**
     * @override
     */
    onSizeChange (): void {
        const label = this.getGraphicItem("label"),
            { width, height } = label && label.props;

        this.shape("shape", {
            width: (this.props.width || width) + this.props.paddingLeft + this.props.paddingRight,
            height: (this.props.height || height) + this.props.paddingTop + this.props.paddingBottom

        }).text("label", {
            text: this.props.label,
            centered: this.props.labelCentered,
            x: this.props.paddingLeft,
            y: this.props.paddingTop
        });

        super.onSizeChange();
    }
}