import { Module } from "./../Module";
import { Util, Color } from "./../Tool";
import { PIXI } from "./../index";
import { ITextProps } from "./../Interface";


export class Text extends Module {

    /* ATTRIBUTES */

    /**
     * Properties of a text
     */
    props: ITextProps;

    /**
     * Special container for the text
     */
    container: PIXI.Text;


    /* CONSTRUCTOR */

    constructor () {
        super();

        this.setProps({
            text        : "",
            align       : "left",
            breakWords  : false,
            dropShadow  : false,
            dropShadowAlpha: 1,
            dropShadowAngle: 80,
            dropShadowColor: Color.black,
            dropShadowDistance: 0,
            dropShadowBlur: 2,
            fill: Color.black,
            fontFamily: "Verdana",
            fontSize: 20,
            fontStyle: "normal",
            fontVariant: "normal",
            fontWeight: "normal",
            letterSpacing: 0,
            lineHeight: null,
            padding: 0,
            stroke: Color.black,
            strokeThickness: 0,
            textBaseline: "alphabetic",
            wordWrap: false,
            wordWrapWidth: 100
        });

        this.container = new PIXI.Text(this.props.text);

        this.connect(this.signals.propChange, "text", () => (this.container.text = this.props.text))
            .connect(this.signals.propChange, ["align", "breakWords", "dropShadow", "dropShadowAlpha", "dropShadowAngle", "dropShadowColor", "dropShadowDistance", "fill", "fontFamily", "fontSize", "fontStyle", "fontVariant",
            "fontWeight", "letterSpacing", "lineHeight", "padding", "stroke", "strokeThickness", "textBaseline", "wordWrap", "wordWrapWidth"], this._onStyleChange.bind(this));
    }

    update (tick) {
        super.update(tick);

        if (this.container) {
            this.props.width    = this.container.width;
            this.props.height   = this.container.height;
        }
    }


    /* EVENTS */

    /**
     * When a font style property has changed
     */
    _onStyleChange (): void {
        const { align, breakWords, dropShadow, dropShadowAlpha, dropShadowAngle, dropShadowBlur, dropShadowColor, dropShadowDistance, fill, fontFamily, fontSize, fontStyle, fontVariant, fontWeight,
            letterSpacing, lineHeight, padding, stroke, strokeThickness, textBaseline, wordWrap, wordWrapWidth } = this.props;

        this.container.style.align = align;
        this.container.style.breakWords = breakWords;
        this.container.style.dropShadow = dropShadow;
        this.container.style.dropShadowAlpha = dropShadowAlpha;
        this.container.style.dropShadowAngle = Util.toRadians(dropShadowAngle);
        this.container.style.dropShadowColor = dropShadowColor;
        this.container.style.dropShadowBlur = dropShadowBlur;
        this.container.style.dropShadowDistance = dropShadowDistance;
        this.container.style.fill = fill;
        this.container.style.fontFamily = fontFamily;
        this.container.style.fontSize = fontSize;
        this.container.style.fontStyle = fontStyle;
        this.container.style.fontVariant = fontVariant;
        this.container.style.fontWeight = fontWeight;
        this.container.style.letterSpacing = letterSpacing;
        this.container.style.lineHeight = lineHeight;
        this.container.style.padding = padding;
        this.container.style.stroke = stroke;
        this.container.style.strokeThickness = strokeThickness;
        this.container.style.textBaseline = textBaseline;
        this.container.style.wordWrap = wordWrap;
        this.container.style.wordWrapWidth = wordWrapWidth;
    }
}